/*
 * abissd/stream.c - abissd, stream management
 *
 * Written 2004 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2004. All rights reserved.
 * For licensing and warranty information, see the file ../../COPYING.LGPLv21
 */


#include <stdlib.h>
#include <stdio.h>

#include "stream.h"


#define SID_FMT "(0x%llx:%lld:0x%llx)"
#define SID_ARGS(s) \
  (unsigned long long) (s)->dev, \
  (unsigned long long) (s)->ino, \
  (unsigned long long) (s)->fid


static struct stream *streams = NULL;


struct stream *find_stream(const struct abissd_id *id)
{
    struct stream *stream;

    for (stream = streams; stream; stream = stream->next)
	if (stream->id.fid == id->fid) {
	    if (stream->id.dev != id->dev || stream->id.ino != id->ino) {
		fprintf(stderr,
		  "find_stream: " SID_FMT " != " SID_FMT "\n",
		  SID_ARGS(id),SID_ARGS(&stream->id));
		abort();
	    }
	    break;
	}
    return stream;
}


struct stream *new_stream(const struct abissd_id *id)
{
    struct stream *new;

    if (find_stream(id)) {
	fprintf(stderr,
	  "WARNING: new_stream: " SID_FMT " is already registered\n",
	  SID_ARGS(id));
	return NULL;
    }
    new = malloc(sizeof(struct stream));
    new->id = *id;
    new->next = streams;
    streams = new;
    return new;
}


void destroy_stream(struct stream *stream)
{
    struct stream **walk;

    for (walk = &streams; *walk != stream; walk = &(*walk)->next);
    *walk = stream->next;
    free(stream);
}
