/*
 * src/chabiss/chabiss.c - Change ABISS scheduler and allocator
 *
 * Written 2004 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2004. All rights reserved.
 * For licensing and warranty information, see the file ../../COPYING.GPLv2
 */


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <abiss_msg.h>


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s mount_point [scheduler_name [allocator_name]]\n",
      name);
    fprintf(stderr,"usage: %s -q file_or_directory\n",name);
    fprintf(stderr,"\n  -q  query current settings\n");
    exit(1);
}


static void set(const char *name,int argc,const char **argv,int fd)
{
    static struct abiss_set_msg msg = { .header = { .type = abiss_set }};

    switch (argc) {
	case 4:
	    if (strlen(argv[3]) >= ABISS_NAME_LENGTH) {
		fprintf(stderr,
		  "%s: allocator name is too long (max %d characters)\n",
		  name,ABISS_NAME_LENGTH-1);
		exit(1);
	    }
	    strcpy(msg.alloc_name,argv[3]);
	    /* fall through */
	case 3:
	    if (strlen(argv[2]) >= ABISS_NAME_LENGTH) {
		fprintf(stderr,
		  "%s: scheduler name is too long (max %d characters)\n",
		  name,ABISS_NAME_LENGTH-1);
		exit(1);
	    }
	    strcpy(msg.sched_name,argv[2]);
	    /* fall through */
	case 2:
	    break;
	default:
	    usage(name);
    }
    if (ioctl(fd,ABISS_IOCTL,&msg) < 0) {
	perror("ABISS_IOCTL (abiss_set)");
	exit(1);
    }
}


static void query(const char *name,int argc,const char **argv,int fd)
{
    static struct abiss_set_msg msg = { .header = { .type = abiss_query }};

    if (ioctl(fd,ABISS_IOCTL,&msg) < 0) {
	perror("ABISS_IOCTL (abiss_query)");
	exit(1);
    }
    printf("scheduler: %s\n",*msg.sched_name ? msg.sched_name : "(none)");
    printf("allocator: %s\n",*msg.alloc_name ? msg.alloc_name : "(none)");
}


int main(int argc,const char **argv)
{
    const char *name = argv[0];
    int q = 0;
    int fd;

    if (argc < 2)
	usage(*argv);
    if (*argv[1] == '-') {
	if (strcmp(argv[1],"-q"))
	    usage(*argv);
	q = 1;
	argv++;
	argc--;
    }
    if (q) {
	fd = open(argv[1],O_RDONLY);
	query(name,argc,argv,fd);
    }
    else {
	if (chdir(argv[1]) < 0) {
	    perror(argv[1]);
	    return 1;
	}
	fd = open(".",O_RDONLY);
	set(name,argc,argv,fd);
    }
    (void) close(fd);
    return 0;
}
